"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.generateOrgData = exports.builder = exports.desc = exports.command = void 0;
const debugLib = require("debug");
const yargs = require("yargs");
const get_logging_path_1 = require("../lib/get-logging-path");
const debug = debugLib('snyk:generate-data-script');
const types_1 = require("../lib/types");
const load_file_1 = require("../load-file");
const generate_targets_data_1 = require("../scripts/generate-targets-data");
exports.command = ['import:data'];
exports.desc = 'Generate data required for targets to be imported via API to create Snyk projects.\n';
exports.builder = {
    orgsData: {
        required: true,
        default: undefined,
        desc: 'Path to organizations data file generated with "orgs:create" command',
    },
    source: {
        required: true,
        default: types_1.SupportedIntegrationTypesImportData.GITHUB,
        choices: [...Object.values(types_1.SupportedIntegrationTypesImportData)],
        desc: 'The source of the targets to be imported e.g. Github, Github Enterprise, Gitlab, Azure. This will be used to make an API call to list all available entities per org',
    },
    sourceUrl: {
        required: false,
        default: undefined,
        desc: 'Custom base url for the source API that can list organizations (e.g. Github Enterprise url)',
    },
};
const entityName = {
    github: 'org',
    'github-enterprise': 'org',
    gitlab: 'group',
    'azure-repos': 'org',
    'bitbucket-server': 'project',
    'bitbucket-cloud': 'workspace',
};
async function generateOrgData(source, orgsData, sourceUrl) {
    try {
        get_logging_path_1.getLoggingPath();
        const content = await load_file_1.loadFile(orgsData);
        let orgsDataJson;
        try {
            const orgsJson = JSON.parse(content);
            orgsDataJson = [...orgsJson.orgData];
        }
        catch (e) {
            throw new Error(`Failed to parse ${entityName[source]}s from ${orgsData}. ERROR: ${e.message}`);
        }
        if (orgsDataJson.length === 0) {
            throw new Error(`No ${entityName[source]}s could be loaded from ${orgsData}.`);
        }
        const res = await generate_targets_data_1.generateTargetsImportDataFile(source, orgsDataJson, sourceUrl);
        const targetsMessage = res.targets.length > 0
            ? `Found ${res.targets.length} ${entityName[source]}(s). Written the data to file: ${res.fileName}`
            : `⚠ No import ${entityName[source]}(s) data generated!`;
        return {
            fileName: res.fileName,
            exitCode: 0,
            message: targetsMessage,
        };
    }
    catch (e) {
        const errorMessage = `ERROR! Failed to generate data. Try running with \`DEBUG=snyk* <command> for more info\`.\nERROR: ${e}`;
        return {
            fileName: undefined,
            exitCode: 1,
            message: errorMessage,
        };
    }
}
exports.generateOrgData = generateOrgData;
async function handler(argv) {
    const { source, orgsData, sourceUrl } = argv;
    debug('ℹ️  Options: ' + JSON.stringify(argv));
    const res = await generateOrgData(source, orgsData, sourceUrl);
    if (res.exitCode === 1) {
        debug('Failed to create organizations.\n' + res.message);
        console.error(res.message);
        setTimeout(() => yargs.exit(1, new Error(res.message)), 3000);
    }
    else {
        console.log(res.message);
    }
}
exports.handler = handler;
//# sourceMappingURL=import:data.js.map