"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.createListImported = exports.builder = exports.desc = exports.command = void 0;
const debugLib = require("debug");
const _ = require("lodash");
const yargs = require("yargs");
const get_logging_path_1 = require("../lib/get-logging-path");
const types_1 = require("../lib/types");
const debug = debugLib('snyk:generate-data-script');
const generate_imported_targets_from_snyk_1 = require("../scripts/generate-imported-targets-from-snyk");
exports.command = ['list:imported'];
exports.desc = 'List all targets imported in Snyk for a given group & source type. An analysis is performed on all current organizations and their projects to generate this. The generated file can be used to skip previously imported targets when running the `import` command';
exports.builder = {
    groupId: {
        required: false,
        default: undefined,
        desc: 'Public id of the group in Snyk (available on group settings)',
    },
    orgId: {
        required: false,
        default: undefined,
        desc: 'Public id of the organization in Snyk (available in organization settings)',
    },
    integrationType: {
        required: true,
        default: [...Object.values(types_1.SupportedIntegrationTypesToListSnykTargets)],
        choices: [...Object.values(types_1.SupportedIntegrationTypesToListSnykTargets)],
        desc: 'The configured integration type (source of the projects in Snyk e.g. Github, Github Enterprise.). This will be used to pick the correct integrationID from each org in Snyk E.g. --integrationType=github, --integrationType=github-enterprise',
    },
};
const entityName = {
    github: 'repo',
    'github-enterprise': 'repo',
    'bitbucket-cloud': 'repo',
    gcr: 'images',
    'docker-hub': 'images',
    gitlab: 'repo',
    'azure-repos': 'repo',
    'bitbucket-server': 'repo',
};
async function createListImported(integrationType, groupId, orgId) {
    try {
        if (!(groupId || orgId)) {
            throw new Error('Missing required parameters: orgId or groupId must be provided.');
        }
        if (groupId && orgId) {
            throw new Error('Too many parameters: orgId or groupId must be provided, not both.');
        }
        const integrationTypes = _.castArray(integrationType);
        const { targets, fileName, failedOrgs } = await generate_imported_targets_from_snyk_1.generateSnykImportedTargets({ groupId, orgId }, integrationTypes);
        const integrationEntity = integrationTypes.length > 1 ? 'target' : entityName[integrationTypes[0]];
        const entityMessage = groupId ? `Group ${groupId} ` : `Org ${orgId}`;
        const targetsMessage = targets.length > 0
            ? `Found ${targets.length} ${integrationEntity}(s). Written the data to file: ${fileName}`
            : `⚠ No ${integrationEntity}(s) ${entityMessage} and integration type(s) ${integrationTypes.join(', ')}!`;
        if (failedOrgs.length > 0) {
            console.warn(`Failed to process the following orgs: ${failedOrgs
                .map((org) => org.id)
                .join(',')}`);
        }
        return {
            fileName: fileName,
            exitCode: 0,
            message: targetsMessage,
        };
    }
    catch (e) {
        const errorMessage = `ERROR! Failed to list imported targets in Snyk. Try running with \`DEBUG=snyk* <command> for more info\`.\nERROR: ${e.message}`;
        return {
            fileName: undefined,
            exitCode: 1,
            message: errorMessage,
        };
    }
}
exports.createListImported = createListImported;
async function handler(argv) {
    get_logging_path_1.getLoggingPath();
    const { groupId, integrationType, orgId } = argv;
    debug('ℹ️  Options: ' + JSON.stringify(argv));
    const res = await createListImported(integrationType, groupId, orgId);
    if (res.exitCode === 1) {
        debug('Failed to create organizations.\n' + res.message);
        console.error(res.message);
        setTimeout(() => yargs.exit(1, new Error(res.message)), 3000);
    }
    else {
        console.log(res.message);
    }
}
exports.handler = handler;
//# sourceMappingURL=list:imported.js.map