"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.createOrg = exports.builder = exports.desc = exports.command = void 0;
const debugLib = require("debug");
const yargs = require("yargs");
const debug = debugLib('snyk:orgs-create-script');
const lib_1 = require("../lib");
const create_orgs_1 = require("../scripts/create-orgs");
exports.command = ['orgs:create'];
exports.desc = 'Create the organizations in Snyk based on data file generated with `orgs:data` command. Output generates key data for created and existing organizations for use to generate project import data.';
exports.builder = {
    file: {
        required: true,
        default: undefined,
        desc: 'Path to data file generated with `orgs:data` command',
    },
    noDuplicateNames: {
        required: false,
        desc: 'Skip creating an organization if the given name is already taken within the Group.',
    },
    includeExistingOrgsInOutput: {
        required: false,
        default: true,
        desc: 'Log existing organization information as well as newly created',
    },
};
async function createOrg(file, includeExistingOrgsInOutput, noDuplicateNames) {
    try {
        lib_1.getLoggingPath();
        const res = await create_orgs_1.createOrgs(file, {
            noDuplicateNames,
            includeExistingOrgsInOutput,
        });
        const orgsMessage = res.orgs.length > 0
            ? `Created ${res.orgs.length} out of ${res.totalOrgs} organization(s). Written the data to file: ${res.fileName}`
            : `⚠ No organization(s) created!`;
        return {
            fileName: res.fileName,
            exitCode: 0,
            message: orgsMessage,
        };
    }
    catch (e) {
        const errorMessage = `ERROR! Failed to create organizations.\nTry running with \`DEBUG=snyk* <command> for more info\`.\nERROR: ${e.message}`;
        return {
            fileName: undefined,
            exitCode: 1,
            message: errorMessage,
        };
    }
}
exports.createOrg = createOrg;
async function handler(argv) {
    const { file, noDuplicateNames, includeExistingOrgsInOutput } = argv;
    debug('ℹ️  Options: ' + JSON.stringify(argv));
    const res = await createOrg(file, includeExistingOrgsInOutput, noDuplicateNames);
    if (res.exitCode === 1) {
        debug('Failed to create organizations.\n' + res.message);
        console.error(res.message);
        setTimeout(() => yargs.exit(1, new Error(res.message)), 3000);
    }
    else {
        console.log(res.message);
    }
}
exports.handler = handler;
//# sourceMappingURL=orgs:create.js.map