"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.generateOrgImportData = exports.builder = exports.desc = exports.command = void 0;
const debugLib = require("debug");
const yargs = require("yargs");
const debug = debugLib('snyk:orgs-data-script');
const get_logging_path_1 = require("../lib/get-logging-path");
const types_1 = require("../lib/types");
const generate_org_data_1 = require("../scripts/generate-org-data");
exports.command = ['orgs:data'];
exports.desc = 'Generate data required for Orgs to be created via API by mirroring a given source.\n';
exports.builder = {
    sourceOrgPublicId: {
        required: false,
        default: undefined,
        desc: 'Public id of the organization in Snyk that can be used as a template to copy all supported organization settings.',
    },
    groupId: {
        required: true,
        default: undefined,
        desc: 'Public id of the group in Snyk (available on group settings)',
    },
    sourceUrl: {
        required: false,
        default: undefined,
        desc: 'Custom base url for the source API that can list organizations (e.g. Github Enterprise url)',
    },
    skipEmptyOrgs: {
        required: false,
        desc: 'Skip any organizations that do not any targets. (e.g. Github Organization does not have any repos)',
    },
    source: {
        required: true,
        default: types_1.SupportedIntegrationTypesImportOrgData.GITHUB,
        choices: [...Object.values(types_1.SupportedIntegrationTypesImportOrgData)],
        desc: 'The source of the targets to be imported e.g. Github, Github Enterprise, Gitlab, Bitbucket Server, Bitbucket Cloud',
    },
};
async function generateOrgImportData(source, groupId, sourceOrgPublicId, sourceUrl, skipEmptyOrgs) {
    try {
        get_logging_path_1.getLoggingPath();
        const res = await generate_org_data_1.generateOrgImportDataFile(source, groupId, sourceOrgPublicId, sourceUrl, skipEmptyOrgs);
        const orgsMessage = res.orgs.length > 0
            ? `Found ${res.orgs.length} ${generate_org_data_1.entityName[source]}(s). Written the data to file: ${res.fileName}`
            : `⚠ No ${generate_org_data_1.entityName[source]}(s) found!`;
        return {
            fileName: res.fileName,
            exitCode: 0,
            message: orgsMessage,
        };
    }
    catch (e) {
        const errorMessage = `ERROR! Failed to generate data. Try running with \`DEBUG=snyk* <command> for more info\`.\nERROR: ${e.message}`;
        return {
            fileName: undefined,
            exitCode: 1,
            message: errorMessage,
        };
    }
}
exports.generateOrgImportData = generateOrgImportData;
async function handler(argv) {
    const { source, sourceOrgPublicId, groupId, sourceUrl, skipEmptyOrgs = false, } = argv;
    debug('ℹ️  Options: ' + JSON.stringify(argv));
    const res = await generateOrgImportData(source, groupId, sourceOrgPublicId, sourceUrl, skipEmptyOrgs);
    if (res.exitCode === 1) {
        debug('Failed to create organizations.\n' + res.message);
        console.error(res.message);
        setTimeout(() => yargs.exit(1, new Error(res.message)), 3000);
    }
    else {
        console.log(res.message);
    }
}
exports.handler = handler;
//# sourceMappingURL=orgs:data.js.map