"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.syncOrg = exports.builder = exports.desc = exports.command = void 0;
const debugLib = require("debug");
const yargs = require("yargs");
const debug = debugLib('snyk:sync-cmd');
const get_logging_path_1 = require("../lib/get-logging-path");
const types_1 = require("../lib/types");
const types_2 = require("../lib/types");
const types_3 = require("../lib/types");
const sync_org_projects_1 = require("../scripts/sync/sync-org-projects");
exports.command = ['sync'];
exports.desc = 'Sync targets (e.g. repos) and their projects between Snyk and SCM for a given organization. Actions include:\n - updating monitored branch in Snyk to match the default branch from SCM';
exports.builder = {
    orgPublicId: {
        required: true,
        default: undefined,
        desc: 'Public id of the organization in Snyk that will be updated',
    },
    sourceUrl: {
        required: false,
        default: undefined,
        desc: 'Custom base url for the source API that can list organizations (e.g. Github Enterprise url)',
    },
    source: {
        required: true,
        default: types_3.SupportedIntegrationTypesUpdateProject.GITHUB,
        choices: [...Object.values(types_3.SupportedIntegrationTypesUpdateProject)],
        desc: 'List of sources to be synced e.g. Github, Github Enterprise, Gitlab, Bitbucket Server, Bitbucket Cloud',
    },
    dryRun: {
        required: false,
        default: false,
        desc: 'Dry run option. Will create a log file listing the potential updates',
    },
    snykProduct: {
        required: false,
        default: types_2.SupportedProductsUpdateProject.OPEN_SOURCE,
        choices: [...Object.values(types_2.SupportedProductsUpdateProject)],
        desc: 'List of Snyk Products to consider when syncing an SCM repo for deleting projects & importing new ones (default branch will be updated for all projects in a target). Monitored Snyk Code repos are automatically synced already, if Snyk Code is enabled any new repo imports will bring in Snyk Code projects',
    },
    exclusionGlobs: {
        required: false,
        default: undefined,
        desc: 'Comma separated list of glob patterns to prevent matching files from being in Snyk. This will refuse to import new files matching the patterns and de-activate existing projects that match these files.',
    },
};
async function syncOrg(source, orgPublicId, sourceUrl, config = {
    dryRun: false,
    entitlements: ['openSource'],
    // TODO: expose manifest types to allow syncing only some projects
}) {
    try {
        get_logging_path_1.getLoggingPath();
        const res = await sync_org_projects_1.updateOrgTargets(orgPublicId, source, sourceUrl, config);
        const nothingToUpdate = res.processedTargets == 0 &&
            res.meta.projects.updated.length == 0 &&
            res.meta.projects.failed.length == 0;
        const orgMessage = nothingToUpdate
            ? `Did not detect any changes to apply`
            : `Processed ${res.processedTargets} targets (${res.failedTargets} failed)\nUpdated ${res.meta.projects.updated.length} projects\nFind more information in ${res.fileName}${res.failedFileName ? ` and ${res.failedFileName}` : ''}`;
        return {
            fileName: res.fileName,
            exitCode: 0,
            message: `Finished syncing all ${source} targets for Snyk organization ${orgPublicId}\n` +
                orgMessage,
        };
    }
    catch (e) {
        const errorMessage = `ERROR! Failed to sync organization. Try running with \`DEBUG=snyk* <command> for more info\`.\nERROR: ${e.message}`;
        return {
            fileName: undefined,
            exitCode: 1,
            message: errorMessage,
        };
    }
}
exports.syncOrg = syncOrg;
async function handler(argv) {
    const { source, orgPublicId, sourceUrl, dryRun = false, snykProduct = [types_2.SupportedProductsUpdateProject.OPEN_SOURCE], exclusionGlobs, } = argv;
    debug('ℹ️  Options: ' + JSON.stringify(argv));
    if (Array.isArray(source)) {
        throw new Error('Please provide a single value for --source. Multiple sources processing is not supported.');
    }
    const sourceList = [];
    sourceList.push(source);
    const manifestTypes = []; // TODO: let users provide this
    const entitlements = [];
    const exclusions = [];
    if (exclusionGlobs) {
        exclusions.push(...(Array.isArray(exclusionGlobs)
            ? exclusionGlobs.join(',').split(',')
            : exclusionGlobs.split(',')));
    }
    const products = Array.isArray(snykProduct) ? snykProduct : [snykProduct];
    for (const p of products) {
        entitlements.push(types_1.productEntitlements[p]);
    }
    console.log(`ℹ️  Running sync for ${source} projects in org: ${orgPublicId} (products to be synced: ${products.join(',')})`);
    const res = await syncOrg(sourceList, orgPublicId, sourceUrl, {
        dryRun,
        entitlements,
        manifestTypes,
        exclusionGlobs: exclusions,
    });
    if (res.exitCode === 1) {
        debug('Failed to sync organizations.\n' + res.message);
        console.error(res.message);
        setTimeout(() => yargs.exit(1, new Error(res.message)), 3000);
    }
    else {
        console.log(res.message);
    }
}
exports.handler = handler;
//# sourceMappingURL=sync.js.map