"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.importTargets = exports.importTarget = void 0;
require("source-map-support/register");
const pMap = require("p-map");
const path = require("path");
const debugLib = require("debug");
const _ = require("lodash");
const get_api_token_1 = require("../../get-api-token");
const get_snyk_host_1 = require("../../get-snyk-host");
const log_imported_targets_1 = require("../../../loggers/log-imported-targets");
const get_logging_path_1 = require("../../get-logging-path");
const log_failed_imports_1 = require("../../../loggers/log-failed-imports");
const log_import_jobs_1 = require("../../../loggers/log-import-jobs");
const get_concurrent_imports_number_1 = require("../../get-concurrent-imports-number");
const common_1 = require("../../../common");
const request_with_rate_limit_1 = require("./request-with-rate-limit");
const debug = debugLib('snyk:api-import');
async function importTarget(requestManager, orgId, integrationId, target, files, exclusionGlobs, loggingPath) {
    var _a;
    const logPath = loggingPath || get_logging_path_1.getLoggingPath();
    get_api_token_1.getApiToken();
    debug('Importing:', JSON.stringify({ orgId, integrationId, target }));
    if (!orgId || !integrationId || Object.keys(target).length === 0) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId, integrationId, target.
      \nFor more information see: https://snyk.docs.apiary.io/#reference/import-projects/import/import-targets`);
    }
    try {
        const body = {
            target: isGitlabTarget(target)
                ? _.pick(target, 'id', 'branch')
                : _.pick(target, ...common_1.targetProps),
            files,
            exclusionGlobs,
        };
        get_snyk_host_1.getSnykHost();
        const res = await request_with_rate_limit_1.requestWithRateLimitHandling(requestManager, `/org/${orgId.trim()}/integrations/${integrationId}/import`, 'post', body);
        const statusCode = res.statusCode || res.status;
        if (!statusCode || statusCode !== 201) {
            throw new Error('Expected a 201 response, instead received: ' +
                JSON.stringify({ data: res.data, statusCode: res.statusCode }));
        }
        const locationUrl = (_a = res.headers) === null || _a === void 0 ? void 0 : _a['location'];
        if (!locationUrl) {
            throw new Error('No import location url returned. Please re-try the import.');
        }
        debug(`Received locationUrl for ${target.name || 'target'}: ${locationUrl}`);
        await log_imported_targets_1.logImportedTargets([{ target, integrationId, orgId }], locationUrl, loggingPath);
        return {
            pollingUrl: locationUrl,
            integrationId,
            target,
            orgId,
        };
    }
    catch (error) {
        const errorBody = error.data || error;
        const errorMessage = errorBody.message;
        const err = new Error('Could not complete API import');
        err.innerError = error;
        console.error(`Failed to kick off import for target: ${JSON.stringify(target)}.\nERROR name: ${error.name} msg: ${errorMessage} See more information in logs located at ${path.join(logPath, orgId)}.${common_1.FAILED_LOG_NAME} or re-start in DEBUG mode.`);
        throw err;
    }
}
exports.importTarget = importTarget;
let failedMoreThanOnce = false;
async function importTargets(requestManager, targets, loggingPath = get_logging_path_1.getLoggingPath()) {
    const pollingUrls = [];
    let failed = 0;
    const concurrentImports = get_concurrent_imports_number_1.getConcurrentImportsNumber();
    await pMap(targets, async (t) => {
        try {
            const { orgId, integrationId, target, files, exclusionGlobs } = t;
            const { pollingUrl } = await importTarget(requestManager, orgId, integrationId, target, files, exclusionGlobs, loggingPath);
            await log_import_jobs_1.logImportJobsPerOrg(orgId, pollingUrl);
            pollingUrls.push(pollingUrl);
        }
        catch (error) {
            failed++;
            const { orgId, integrationId, target } = t;
            await log_failed_imports_1.logFailedImports(orgId, integrationId, target, { errorMessage: error.message, innerError: error.innerError }, loggingPath);
            if (failed % concurrentImports === 0) {
                if (failedMoreThanOnce) {
                    console.error(`Every import in the last few batches failed, stopping as this is unexpected! Please check if everything is configured ok and review the logs located at ${loggingPath}/*. If everything looks OK re-start the import, previously imported targets will be skipped.`);
                    // die immediately
                    process.exit(1);
                }
                failedMoreThanOnce = true;
            }
        }
    }, { concurrency: get_concurrent_imports_number_1.getConcurrentImportsNumber() });
    return _.uniq(pollingUrls);
}
exports.importTargets = importTargets;
function isGitlabTarget(target) {
    const keys = Object.keys(target);
    if (keys.find((k) => k === 'id') && keys.find((k) => k === 'branch')) {
        return true;
    }
    return false;
}
//# sourceMappingURL=index.js.map