"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSnykTarget = exports.listAllSnykTargets = exports.listTargets = exports.listProjects = exports.deleteOrg = exports.setNotificationPreferences = exports.listIntegrations = void 0;
require("source-map-support/register");
const debugLib = require("debug");
const qs = require("querystring");
const get_api_token_1 = require("../../get-api-token");
const get_snyk_host_1 = require("../../get-snyk-host");
const debug = debugLib('snyk:api-group');
async function listIntegrations(requestManager, orgId) {
    get_api_token_1.getApiToken();
    get_snyk_host_1.getSnykHost();
    debug('Listing integrations for org: ' + orgId);
    if (!orgId) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId.
      \nFor more information see: https://snyk.docs.apiary.io/#reference/integrations/list`);
    }
    const res = await requestManager.request({
        verb: 'get',
        url: `/org/${orgId.trim()}/integrations`,
        body: JSON.stringify({}),
    });
    const statusCode = res.statusCode || res.status;
    if (!statusCode || statusCode !== 200) {
        throw new Error('Expected a 200 response, instead received: ' +
            JSON.stringify({ data: res.data || res.body, status: statusCode }));
    }
    return res.data || {};
}
exports.listIntegrations = listIntegrations;
const defaultDisabledSettings = {
    'new-issues-remediations': {
        enabled: false,
        issueType: 'none',
        issueSeverity: 'high',
    },
    'project-imported': {
        enabled: false,
    },
    'test-limit': {
        enabled: false,
    },
    'weekly-report': {
        enabled: false,
    },
};
async function setNotificationPreferences(requestManager, orgId, orgName, settings = defaultDisabledSettings) {
    get_api_token_1.getApiToken();
    get_snyk_host_1.getSnykHost();
    debug(`Disabling notifications for org: ${orgName} (${orgId})`);
    if (!orgId) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId, settings.
      \nFor more information see: https://snyk.docs.apiary.io/#reference/organizations/the-snyk-organization-for-a-request/set-notification-settings`);
    }
    try {
        const res = await requestManager.request({
            verb: 'put',
            url: `/org/${orgId.trim()}/notification-settings`,
            body: JSON.stringify(settings),
        });
        const statusCode = res.statusCode || res.status;
        if (!statusCode || statusCode !== 200) {
            throw new Error('Expected a 200 response, instead received: ' +
                JSON.stringify({ data: res.data, status: statusCode }));
        }
        return res.data || {};
    }
    catch (e) {
        debug('Failed to update notification settings for ', orgId, e);
        throw e;
    }
}
exports.setNotificationPreferences = setNotificationPreferences;
async function deleteOrg(requestManager, orgId) {
    get_api_token_1.getApiToken();
    get_snyk_host_1.getSnykHost();
    debug(`Deleting org: "${orgId}"`);
    if (!orgId) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId.
      \nFor more information see: https://snyk.docs.apiary.io/#reference/organizations/manage-organization/remove-organization`);
    }
    const res = await requestManager.request({
        verb: 'delete',
        url: `/org/${orgId}`,
        body: JSON.stringify({}),
    });
    const statusCode = res.statusCode || res.status;
    if (!statusCode || statusCode !== 204) {
        throw new Error('Expected a 204 response, instead received: ' +
            JSON.stringify({ data: res.data, status: statusCode }));
    }
    return res.data;
}
exports.deleteOrg = deleteOrg;
async function listProjects(requestManager, orgId, filters) {
    get_api_token_1.getApiToken();
    get_snyk_host_1.getSnykHost();
    debug(`Listing all projects for org: ${orgId} with filter ${JSON.stringify(filters)}`);
    if (!orgId) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId, settings.
        \nFor more information see: https://snyk.docs.apiary.io/#reference/projects/all-projects/list-all-projects`);
    }
    const projects = await listAllProjects(requestManager, orgId, filters);
    return {
        org: {
            id: orgId,
        },
        projects: projects,
    };
}
exports.listProjects = listProjects;
async function listAllProjects(requestManager, orgId, filters) {
    let lastPage = false;
    const projectsList = [];
    let pageCount = 1;
    let nextPageLink = undefined;
    while (!lastPage) {
        try {
            debug(`Fetching page ${pageCount} to list all projects for org ${orgId}\n`);
            const { projects, next, } = await getProjectsPage(requestManager, orgId, filters, nextPageLink);
            projectsList.push(...projects);
            next
                ? ((lastPage = false), (nextPageLink = next))
                : ((lastPage = true), (nextPageLink = ''));
            pageCount++;
        }
        catch (e) {
            debug('Failed to get projects for ', orgId, e);
            throw e;
        }
    }
    return projectsList;
}
async function getProjectsPage(requestManager, orgId, filters, nextPageLink) {
    const query = qs.stringify({
        version: '2022-09-15~beta',
        ...filters,
    });
    const url = nextPageLink !== null && nextPageLink !== void 0 ? nextPageLink : `/orgs/${orgId.trim()}/projects?${query}`;
    const res = await requestManager.request({
        verb: 'get',
        url: url,
        useRESTApi: true,
    });
    const statusCode = res.statusCode || res.status;
    if (!statusCode || statusCode !== 200) {
        throw new Error('Expected a 200 response, instead received: ' +
            JSON.stringify({ data: res.data, status: statusCode }));
    }
    const response = res.data;
    const projects = convertToSnykProject(response.data);
    const next = response.links.next;
    return { projects, next };
}
function convertToSnykProject(projectData) {
    const projects = [];
    for (const project of projectData) {
        const projectTmp = {
            id: project.id,
            branch: project.attributes.targetReference,
            created: project.attributes.created,
            origin: project.attributes.origin,
            name: project.attributes.name,
            type: project.attributes.type,
            status: project.attributes.status,
        };
        projects.push(projectTmp);
    }
    return projects;
}
async function listTargets(requestManager, orgId, config) {
    get_api_token_1.getApiToken();
    get_snyk_host_1.getSnykHost();
    debug(`Listing all targets for org: ${orgId}`);
    if (!orgId && orgId.length > 0) {
        throw new Error(`Missing required parameters. Please ensure you have set: orgId, settings.
        \nFor more information see: https://apidocs.snyk.io/?version=2022-09-15%7Ebeta#get-/orgs/-org_id-/targets`);
    }
    const targets = await listAllSnykTargets(requestManager, orgId, config);
    return { targets };
}
exports.listTargets = listTargets;
async function listAllSnykTargets(requestManager, orgId, config) {
    let lastPage = false;
    const targetsList = [];
    let pageCount = 1;
    let nextPageLink = undefined;
    while (!lastPage) {
        try {
            debug(`Fetching page ${pageCount} of targets for orgId: ${orgId}\n`);
            const { targets, next } = await getSnykTarget(requestManager, orgId, nextPageLink, config);
            targetsList.push(...targets);
            next
                ? ((lastPage = false), (nextPageLink = next))
                : ((lastPage = true), (nextPageLink = undefined));
            pageCount++;
        }
        catch (e) {
            debug('Failed to get targets for ', orgId, e);
            throw e;
        }
    }
    return targetsList;
}
exports.listAllSnykTargets = listAllSnykTargets;
async function getSnykTarget(requestManager, orgId, nextPageLink, config = {
    limit: 20,
    excludeEmpty: true,
}) {
    const query = qs.stringify({
        version: '2022-09-15~beta',
        ...config,
    });
    const url = nextPageLink !== null && nextPageLink !== void 0 ? nextPageLink : `/orgs/${orgId.trim()}/targets?${query}`;
    const res = await requestManager.request({
        verb: 'get',
        url: url,
        body: undefined,
        useRESTApi: true,
    });
    const statusCode = res.statusCode || res.status;
    if (!statusCode || statusCode !== 200) {
        throw new Error('Expected a 200 response, instead received: ' +
            JSON.stringify({ data: res.data, status: statusCode }));
    }
    const responseData = res.data;
    const targets = responseData.data;
    const { next } = responseData.links;
    return { targets, next };
}
exports.getSnykTarget = getSnykTarget;
//# sourceMappingURL=index.js.map