"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pollImportUrls = exports.pollImportUrl = void 0;
require("source-map-support/register");
const url = require("url");
const sleep = require("sleep-promise");
const debugLib = require("debug");
const _ = require("lodash");
const pMap = require("p-map");
const get_api_token_1 = require("../../get-api-token");
const log_failed_projects_1 = require("../../../loggers/log-failed-projects");
const log_failed_polls_1 = require("../../../loggers/log-failed-polls");
const log_imported_projects_1 = require("../../../loggers/log-imported-projects");
const log_job_result_1 = require("../../../loggers/log-job-result");
const debug = debugLib('snyk:poll-import');
const MIN_RETRY_WAIT_TIME = 20000;
const MAX_RETRY_COUNT = 1000;
async function pollImportUrl(requestManager, locationUrl, retryCount = MAX_RETRY_COUNT, retryWaitTime = MIN_RETRY_WAIT_TIME) {
    get_api_token_1.getApiToken();
    debug(`Polling locationUrl=${locationUrl}`);
    if (!locationUrl) {
        throw new Error(`Missing required parameters. Please ensure you have provided: location url.
      \nFor more information see: https://snyk.docs.apiary.io/#reference/import-projects/import/import-targets`);
    }
    try {
        const { pathname = '' } = url.parse(locationUrl);
        const res = await requestManager.request({
            verb: 'get',
            url: pathname.split('/api/v1/')[1],
            body: JSON.stringify({}),
        });
        const importStatus = res.data;
        const statusCode = res.statusCode || res.status;
        if (!statusCode || statusCode !== 200) {
            throw new Error('Expected a 200 response, instead received: ' +
                JSON.stringify({ data: res.data, status: statusCode }));
        }
        debug(`Import task status is "${importStatus.status}"`);
        if (importStatus.status &&
            importStatus.status !== 'complete' &&
            retryCount > 0) {
            await sleep(retryWaitTime);
            debug(`Will re-check import task in "${retryWaitTime} ms"`);
            return await pollImportUrl(requestManager, locationUrl, --retryCount);
        }
        const projects = [];
        importStatus.logs.forEach((log) => {
            projects.push(...log.projects);
        });
        await log_job_result_1.logJobResult(locationUrl, importStatus);
        return { projects };
    }
    catch (error) {
        console.error(`Could not get status update from import job: ${locationUrl}\n ERROR: ${error.message}`);
        const err = new Error('Could not poll Url');
        err.innerError = error;
        throw err;
    }
}
exports.pollImportUrl = pollImportUrl;
async function pollImportUrls(requestManager, locationUrls) {
    if (!locationUrls) {
        throw new Error('Missing required parameters. Please ensure you have provided: locationUrls.');
    }
    const uniqueLocationUrls = _.uniq(locationUrls);
    const projectsArray = [];
    const allFailedProjects = [];
    await pMap(uniqueLocationUrls, async (locationUrl) => {
        try {
            const importJobId = locationUrl.split('import/')[1];
            console.log(`Checking status for import job id: ${importJobId}`);
            const res = await pollImportUrl(requestManager, locationUrl);
            const [failedProjects, projects] = _.partition(res.projects, (p) => !p.success);
            console.log(`Discovered ${_.uniqBy(projects, 'projectUrl').length} projects from import job id: ${importJobId}${_.uniqBy(projects, 'projectUrl').length
                ? `. ${failedProjects.length} project(s) failed to finish importing.`
                : ''}`);
            allFailedProjects.push(...failedProjects.map((project) => ({
                ...project,
                locationUrl,
            })));
            await log_imported_projects_1.logImportedProjects(locationUrl, projects);
            projectsArray.push(...projects);
        }
        catch (error) {
            await log_failed_polls_1.logFailedPollUrls(locationUrl, {
                errorMessage: _.get(error, 'innerError.message') ||
                    error.innerError ||
                    error.message ||
                    error,
            });
        }
    }, { concurrency: 10 });
    await log_failed_projects_1.logFailedProjects(allFailedProjects);
    return { projects: projectsArray, failed: allFailedProjects };
}
exports.pollImportUrls = pollImportUrls;
//# sourceMappingURL=index.js.map