"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.listAzureProjects = void 0;
const qs = require("querystring");
const base64 = require("base-64");
const debugLib = require("debug");
const get_azure_token_1 = require("./get-azure-token");
const get_base_url_1 = require("./get-base-url");
const request_with_rate_limit_1 = require("../../request-with-rate-limit");
const limiters_1 = require("../../limiters");
const debug = debugLib('snyk:azure');
async function fetchAllProjects(orgName, host) {
    const projectList = [];
    let hasMorePages = true;
    let displayPageNumber = 1;
    let nextPage;
    while (hasMorePages) {
        debug(`Fetching page ${displayPageNumber} for Azure projects`);
        try {
            const { projects, continueFrom } = await getProjects(orgName, host, nextPage);
            if (continueFrom && projects.length > 0) {
                nextPage = continueFrom;
                displayPageNumber++;
            }
            else {
                hasMorePages = false;
            }
            projects.map((project) => {
                const { name, id } = project;
                if (name && id) {
                    projectList.push({ id, name });
                }
            });
        }
        catch (err) {
            throw new Error(JSON.stringify(err));
        }
    }
    return projectList;
}
async function getProjects(orgName, host, continueFrom) {
    const azureToken = get_azure_token_1.getAzureToken();
    const params = {
        stateFilter: 'wellFormed',
        continuationToken: continueFrom,
        'api-version': '4.1',
    };
    const query = qs.stringify(params);
    const headers = {
        Authorization: `Basic ${base64.encode(':' + azureToken)}`,
    };
    const limiter = await limiters_1.limiterForScm(1, 500);
    const { body, statusCode, headers: responseHeaders, } = await request_with_rate_limit_1.limiterWithRateLimitRetries('get', `${host}/${orgName}/_apis/projects?${query}`, headers, limiter, 60000);
    if (statusCode != 200) {
        throw new Error(`Failed to fetch projects for ${host}/${orgName}/_apis/projects?${query}\n
    Status Code: ${statusCode}\n
    Response body: ${JSON.stringify(body)}`);
    }
    const { value: projects } = body;
    return {
        projects,
        continueFrom: responseHeaders['x-ms-continuationtoken'],
    };
}
async function listAzureProjects(azureOrgName, host) {
    debug(`Fetching all projects data for org: ${azureOrgName}`);
    const baseUrl = get_base_url_1.getBaseUrl(host);
    const projects = await fetchAllProjects(azureOrgName, baseUrl);
    return projects;
}
exports.listAzureProjects = listAzureProjects;
//# sourceMappingURL=list-projects.js.map