"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.githubOrganizations = exports.githubEnterpriseOrganizations = exports.listGithubOrgs = void 0;
const rest_1 = require("@octokit/rest");
const parseLinkHeader = require("parse-link-header");
const debugLib = require("debug");
const github_base_url_1 = require("./github-base-url");
const get_github_token_1 = require("./get-github-token");
const debug = debugLib('snyk:github');
async function fetchOrgsForPage(octokit, pageNumber = 1, since = 0, isGithubEnterprise = false) {
    const orgsData = [];
    const params = {
        per_page: 100,
        page: pageNumber,
        since,
    };
    const res = isGithubEnterprise
        ? await octokit.orgs.list(params)
        : await octokit.orgs.listForAuthenticatedUser(params);
    const links = parseLinkHeader(res.headers.link) || {};
    const orgs = res && res.data;
    if (orgs.length) {
        orgsData.push(...orgs.map((org) => ({
            name: org.login,
            id: org.id,
            url: org.url,
        })));
    }
    return {
        orgs: orgsData,
        hasNextPage: !!links.next,
        since: links.next ? Number(links.next.since) : undefined,
    };
}
async function fetchAllOrgs(octokit, page = 0, isGithubEnterprise = false) {
    const orgsData = [];
    let currentPage = page;
    let hasMorePages = true;
    let currentSince = 0;
    while (hasMorePages) {
        currentPage = currentPage + 1;
        debug(`Fetching page ${currentPage}`);
        const { orgs, hasNextPage, since } = await fetchOrgsForPage(octokit, currentPage, currentSince, isGithubEnterprise);
        orgsData.push(...orgs);
        hasMorePages = hasNextPage;
        if (since) {
            currentSince = since;
        }
    }
    return orgsData;
}
async function listGithubOrgs(host) {
    const githubToken = get_github_token_1.getGithubToken();
    const baseUrl = github_base_url_1.getGithubBaseUrl(host);
    const octokit = new rest_1.Octokit({ baseUrl, auth: githubToken });
    debug('Fetching all Github organizations data');
    const orgs = await fetchAllOrgs(octokit, undefined, !!host);
    return orgs;
}
exports.listGithubOrgs = listGithubOrgs;
async function githubEnterpriseOrganizations(sourceUrl) {
    if (!sourceUrl) {
        console.warn('No `sourceUrl` provided for Github Enterprise source, defaulting to https://api.github.com');
    }
    const ghOrgs = await listGithubOrgs(sourceUrl);
    return ghOrgs;
}
exports.githubEnterpriseOrganizations = githubEnterpriseOrganizations;
async function githubOrganizations() {
    const ghOrgs = await listGithubOrgs();
    return ghOrgs;
}
exports.githubOrganizations = githubOrganizations;
//# sourceMappingURL=list-organizations.js.map