"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.listGithubRepos = exports.fetchReposForPage = void 0;
const rest_1 = require("@octokit/rest");
const plugin_retry_1 = require("@octokit/plugin-retry");
const debugLib = require("debug");
const get_github_token_1 = require("./get-github-token");
const github_base_url_1 = require("./github-base-url");
const debug = debugLib('snyk:list-repos-script');
const githubClient = rest_1.Octokit.plugin(plugin_retry_1.retry);
async function fetchReposForPage(octokit, orgName, pageNumber = 1, perPage = 100) {
    var _a, _b;
    const repoData = [];
    const params = {
        per_page: perPage,
        page: pageNumber,
        org: orgName,
    };
    const res = await octokit.repos.listForOrg(params);
    const repos = res && res.data;
    const hasNextPage = repos.length ? true : false;
    debug(`Received # repos from API for page ${pageNumber}: ${repos.length}`);
    for (const repo of repos) {
        if (repo.archived) {
            debug(`Skipping archived repo name: ${repo.name} owner: ${(_a = repo.owner) === null || _a === void 0 ? void 0 : _a.login}`);
            continue;
        }
        repoData.push({
            fork: repo.fork,
            name: repo.name,
            owner: (_b = repo.owner) === null || _b === void 0 ? void 0 : _b.login,
            branch: repo.default_branch,
        });
    }
    return { repos: repoData, hasNextPage };
}
exports.fetchReposForPage = fetchReposForPage;
async function fetchAllRepos(octokit, orgName, page = 1) {
    const repoData = [];
    const MAX_RETRIES = 3;
    let currentPage = page;
    let hasMorePages = true;
    let retries = 0;
    while (hasMorePages) {
        try {
            debug(`Fetching page: ${currentPage}`);
            const { repos, hasNextPage } = await fetchReposForPage(octokit, orgName, currentPage);
            retries = 0;
            currentPage = currentPage + 1;
            hasMorePages = hasNextPage;
            repoData.push(...repos);
        }
        catch (e) {
            debug(`Failed to fetch page: ${currentPage}`, e);
            if ([403, 500, 502].includes(e.status) && retries < MAX_RETRIES) {
                const sleepTime = 120000 * retries; // 2 mins x retry attempt
                retries = retries + 1;
                console.error(`Sleeping for ${sleepTime} ms`);
                await new Promise((r) => setTimeout(r, sleepTime));
            }
            else {
                throw e;
            }
        }
    }
    return repoData;
}
async function listGithubRepos(orgName, host) {
    const githubToken = get_github_token_1.getGithubToken();
    const baseUrl = github_base_url_1.getGithubBaseUrl(host);
    const octokit = new githubClient({ baseUrl, auth: githubToken });
    debug(`Fetching all repos data for org: ${orgName}`);
    const repos = await fetchAllRepos(octokit, orgName);
    return repos;
}
exports.listGithubRepos = listGithubRepos;
//# sourceMappingURL=list-repos.js.map