"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createOrgs = void 0;
const debugLib = require("debug");
const path = require("path");
const fs = require("fs");
const lib_1 = require("../lib");
const lib_2 = require("./../lib");
const org_1 = require("../lib/api/org");
const snyk_request_manager_1 = require("snyk-request-manager");
const log_created_org_1 = require("../loggers/log-created-org");
const write_file_1 = require("../write-file");
const common_1 = require("../common");
const log_failed_org_1 = require("../loggers/log-failed-org");
const stream_data_1 = require("../stream-data");
const debug = debugLib('snyk:create-orgs-script');
async function saveCreatedOrgData(orgData) {
    const fileName = 'snyk-created-orgs.json';
    await write_file_1.writeFile(fileName, { orgData });
    return fileName;
}
async function createNewOrgs(loggingPath, requestManager, groupId, orgsToCreate) {
    const failed = [];
    const created = [];
    for (const orgData of orgsToCreate) {
        const { name, sourceOrgId } = orgData;
        try {
            debug(`Creating new "${name}" organization`);
            const org = await lib_1.createOrg(requestManager, groupId, name, sourceOrgId);
            debug(`Creating new "${name}" organization`);
            debug(`Listing integrations for new "${name}" organization`);
            const integrations = (await org_1.listIntegrations(requestManager, org.id)) || {};
            debug(`Setting notification settings for new "${name}" organization`);
            await org_1.setNotificationPreferences(requestManager, org.id, org.name);
            created.push({
                ...org,
                orgId: org.id,
                integrations,
                groupId,
                origName: name,
                sourceOrgId,
            });
            await log_created_org_1.logCreatedOrg(groupId, name, org, integrations, loggingPath);
        }
        catch (e) {
            failed.push({ groupId, name, sourceOrgId });
            const errorMessage = e.data ? e.data.message : e.message;
            await log_failed_org_1.logFailedOrg(groupId, name, errorMessage || 'Failed to create org, please try again in DEBUG mode.');
            debug(`Failed to create organization with data: ${JSON.stringify(orgData)}`, e);
        }
    }
    return { failed, created };
}
async function listExistingOrgsData(requestManager, existingOrgs) {
    const previouslyCreated = [];
    for (const orgData of existingOrgs) {
        const { name, id, group } = orgData;
        try {
            const integrations = (await org_1.listIntegrations(requestManager, id)) || {};
            previouslyCreated.push({
                ...orgData,
                name,
                orgId: id,
                integrations,
                groupId: group.id,
                origName: name,
            });
        }
        catch (e) {
            debug(`Failed to list integrations for Org: ${orgData.name} (${orgData.id})`, e);
        }
    }
    return { existing: previouslyCreated };
}
async function createOrgs(filePath, options = {
    noDuplicateNames: false,
    includeExistingOrgsInOutput: true,
}, loggingPath = lib_2.getLoggingPath()) {
    const { includeExistingOrgsInOutput, noDuplicateNames } = options;
    const failedOrgs = [];
    let orgsData;
    const orgsFilePath = path.resolve(process.cwd(), filePath);
    if (!fs.existsSync(orgsFilePath)) {
        throw new Error(`File not found ${orgsFilePath}`);
    }
    try {
        orgsData = await stream_data_1.streamData(orgsFilePath, 'orgs');
    }
    catch (e) {
        throw new Error(`Failed to parse organizations from ${filePath}`);
    }
    debug(`Loaded ${orgsData.length} organizations to create ${Date.now()}`);
    const orgsPerGroup = {};
    orgsData.forEach((org) => {
        const { groupId } = org;
        if (!orgsPerGroup[groupId]) {
            orgsPerGroup[groupId] = [org];
        }
        else {
            orgsPerGroup[groupId].push(org);
        }
    });
    const createdOrgs = [];
    const allExistingOrgs = [];
    const requestManager = new snyk_request_manager_1.requestsManager({
        userAgentPrefix: 'snyk-api-import:create-orgs',
    });
    for (const groupId in orgsPerGroup) {
        let orgsToCreate = orgsPerGroup[groupId];
        debug(`Finding existing organizations in group ${groupId}`);
        const { newOrgs, existingOrgs } = await separateExistingOrganizations(loggingPath, requestManager, groupId, orgsToCreate);
        allExistingOrgs.push(...existingOrgs);
        debug(`Found ${existingOrgs.length} existing organizations`);
        if (noDuplicateNames) {
            let duplicates = 0;
            const uniqueOrgNames = new Set(existingOrgs.map((org) => org.name));
            for (const org of orgsToCreate) {
                const { name } = org;
                if (uniqueOrgNames.has(name)) {
                    duplicates += 1;
                    failedOrgs.push(org);
                    await log_failed_org_1.logFailedOrg(groupId, name, 'Refusing to create a duplicate organization with option --noDuplicateNames enabled.');
                }
            }
            debug(`Skipping ${duplicates} duplicate organizations`);
            orgsToCreate = newOrgs;
        }
        debug(`Creating ${orgsToCreate.length} new organizations`);
        const { failed, created } = await createNewOrgs(loggingPath, requestManager, groupId, orgsToCreate);
        failedOrgs.push(...failed);
        createdOrgs.push(...created);
    }
    if (createdOrgs.length === 0 && !includeExistingOrgsInOutput) {
        throw new Error(`All requested organizations failed to be created. Review the errors in ${path.resolve(__dirname, loggingPath)}/<groupId>.${common_1.FAILED_ORG_LOG_NAME}`);
    }
    debug(`Getting existing ${allExistingOrgs.length} orgs data`);
    const allOrgs = [...createdOrgs];
    let existing = [];
    if (includeExistingOrgsInOutput) {
        const res = await listExistingOrgsData(requestManager, allExistingOrgs);
        existing = res.existing;
        allOrgs.push(...existing);
    }
    const fileName = await saveCreatedOrgData(allOrgs);
    return {
        orgs: createdOrgs,
        existing,
        failed: failedOrgs,
        fileName,
        totalOrgs: orgsData.length,
    };
}
exports.createOrgs = createOrgs;
async function separateExistingOrganizations(loggingPath, requestManager, groupId, orgsPerGroup) {
    try {
        return await lib_1.filterOutExistingOrgs(requestManager, orgsPerGroup, groupId);
    }
    catch (e) {
        for (const org of orgsPerGroup) {
            const { name } = org;
            await log_failed_org_1.logFailedOrg(groupId, name, e.data
                ? e.data.message
                : e.message ||
                    'Failed to create org, please try again in DEBUG mode.');
        }
        throw new Error(`All requested organizations failed to be created. Review the errors in ${path.resolve(__dirname, loggingPath)}/<groupId>.${common_1.FAILED_ORG_LOG_NAME}`);
    }
}
//# sourceMappingURL=create-orgs.js.map