"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSnykImportedTargets = exports.targetGenerators = exports.imageProjectToTarget = exports.gitlabProjectToImportLogTarget = exports.bitbucketServerProjectToTarget = exports.projectToTarget = void 0;
const snyk_request_manager_1 = require("snyk-request-manager");
const debugLib = require("debug");
const path = require("path");
const pMap = require("p-map");
const _ = require("lodash");
const types_1 = require("../lib/types");
const lib_1 = require("../lib");
const log_imported_targets_1 = require("../loggers/log-imported-targets");
const common_1 = require("../common");
const generate_target_id_1 = require("../generate-target-id");
const debug = debugLib('snyk:generate-snyk-imported-targets');
function projectToTarget(project) {
    const [owner, name] = project.name.split(':')[0].split('/');
    return {
        owner,
        branch: project.branch || undefined,
        name: name.split('(')[0],
    };
}
exports.projectToTarget = projectToTarget;
function bitbucketServerProjectToTarget(project) {
    const [projectKey, repoSlug] = project.name.split(':')[0].split('/');
    return {
        projectKey,
        repoSlug: repoSlug.split('(')[0],
    };
}
exports.bitbucketServerProjectToTarget = bitbucketServerProjectToTarget;
function gitlabProjectToImportLogTarget(project) {
    // Gitlab target is only `id` & branch and the Snyk API does not return the id.
    // However we are already logging `name` which for Gitlab is "owner/repo", branch & id so if we use the same name we can match on it
    // TODO: add support for customBranch projects
    const name = project.name.split(':')[0];
    return {
        branch: project.branch || undefined,
        name,
    };
}
exports.gitlabProjectToImportLogTarget = gitlabProjectToImportLogTarget;
function imageProjectToTarget(project) {
    return {
        name: project.name,
    };
}
exports.imageProjectToTarget = imageProjectToTarget;
exports.targetGenerators = {
    [types_1.SupportedIntegrationTypesToListSnykTargets.GITHUB]: projectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.GITLAB]: gitlabProjectToImportLogTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.GHE]: projectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.BITBUCKET_CLOUD]: projectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.GCR]: imageProjectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.DOCKER_HUB]: imageProjectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.AZURE_REPOS]: projectToTarget,
    [types_1.SupportedIntegrationTypesToListSnykTargets.BITBUCKET_SERVER]: bitbucketServerProjectToTarget,
};
async function generateSnykImportedTargets(id, integrationTypes) {
    const { groupId, orgId } = id;
    if (!(groupId || orgId)) {
        throw new Error('Missing required parameters: orgId or groupId must be provided.');
    }
    if (groupId && orgId) {
        throw new Error('Too many parameters: orgId or groupId must be provided, not both.');
    }
    const requestManager = new snyk_request_manager_1.requestsManager({
        userAgentPrefix: 'snyk-api-import:list',
    });
    const targetsData = [];
    const groupOrgs = groupId
        ? await lib_1.getAllOrgs(requestManager, groupId)
        : [{ id: orgId }];
    const failedOrgs = [];
    const projectFilters = integrationTypes.length > 1
        ? { limit: 100 }
        : { origin: integrationTypes[0], limit: 100 };
    await pMap(groupOrgs, async (org) => {
        const { id: orgId, name, slug } = org;
        try {
            const [resProjects, resIntegrations] = await Promise.all([
                lib_1.listProjects(requestManager, orgId, projectFilters),
                lib_1.listIntegrations(requestManager, orgId),
            ]);
            const { projects } = resProjects;
            const scmTargets = projects
                .filter((p) => integrationTypes.includes(p.origin))
                .map((p) => {
                const target = exports.targetGenerators[p.origin](p);
                return {
                    target,
                    integrationId: resIntegrations[p.origin],
                };
            });
            const uniqueTargets = new Set();
            const orgTargets = [];
            if (!scmTargets.length || scmTargets.length === 0) {
                console.warn('No projects in org', orgId);
                return;
            }
            for (const data of scmTargets) {
                const { target, integrationId } = data;
                const targetId = generate_target_id_1.generateTargetId(orgId, integrationId, target);
                if (uniqueTargets.has(targetId)) {
                    continue;
                }
                uniqueTargets.add(targetId);
                const importedTarget = {
                    target: _.pick(target, ...common_1.targetProps),
                    integrationId,
                    orgId,
                };
                targetsData.push(importedTarget);
                orgTargets.push(target);
            }
            console.log('Extracted', uniqueTargets.size, 'unique targets from', scmTargets.length, 'projects from org', orgId);
            await log_imported_targets_1.logImportedTargets(targetsData, null, undefined, 'Target exists in Snyk');
        }
        catch (e) {
            failedOrgs.push(org);
            console.warn(`Failed to process projects for organization ${name && slug ? `${name}(${slug})` : orgId}. Continuing.`);
        }
    }, { concurrency: 15 });
    if (targetsData.length === 0) {
        debug('No targets could be generated. Could Snyk Group have no projects?');
        const message = groupId
            ? `Could Snyk organizations in the group (${groupId}) be empty?`
            : `Could the organization ${orgId} be empty?`;
        console.warn(`No targets could be generated. ${message}`);
    }
    return {
        targets: targetsData,
        fileName: path.resolve(lib_1.getLoggingPath(), common_1.IMPORT_LOG_NAME),
        failedOrgs,
    };
}
exports.generateSnykImportedTargets = generateSnykImportedTargets;
//# sourceMappingURL=generate-imported-targets-from-snyk.js.map