"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateOrgImportDataFile = exports.entityName = void 0;
const pMap = require("p-map");
const github_1 = require("../lib/source-handlers/github");
const gitlab_1 = require("../lib/source-handlers/gitlab");
const bitbucket_cloud_1 = require("../lib/source-handlers/bitbucket-cloud/");
const bitbucket_server_1 = require("../lib/source-handlers/bitbucket-server/");
const types_1 = require("../lib/types");
const write_file_1 = require("../write-file");
const sourceGenerators = {
    [types_1.SupportedIntegrationTypesImportOrgData.GITLAB]: gitlab_1.listGitlabGroups,
    [types_1.SupportedIntegrationTypesImportOrgData.GITHUB]: github_1.githubOrganizations,
    [types_1.SupportedIntegrationTypesImportOrgData.GHE]: github_1.githubEnterpriseOrganizations,
    [types_1.SupportedIntegrationTypesImportOrgData.BITBUCKET_SERVER]: bitbucket_server_1.listBitbucketServerProjects,
    [types_1.SupportedIntegrationTypesImportOrgData.BITBUCKET_CLOUD]: bitbucket_cloud_1.listBitbucketCloudWorkspaces,
};
const sourceNotEmpty = {
    [types_1.SupportedIntegrationTypesImportOrgData.GITHUB]: github_1.githubOrganizationIsEmpty,
    [types_1.SupportedIntegrationTypesImportOrgData.GHE]: github_1.githubOrganizationIsEmpty,
    [types_1.SupportedIntegrationTypesImportOrgData.GITLAB]: gitlab_1.gitlabGroupIsEmpty,
    [types_1.SupportedIntegrationTypesImportOrgData.BITBUCKET_SERVER]: bitbucket_server_1.bitbucketServerProjectIsEmpty,
    [types_1.SupportedIntegrationTypesImportOrgData.BITBUCKET_CLOUD]: bitbucket_cloud_1.bitbucketCloudWorkspaceIsEmpty,
};
exports.entityName = {
    github: 'organization',
    'github-enterprise': 'organization',
    gitlab: 'group',
    'bitbucket-server': 'project',
    'bitbucket-cloud': 'workspace',
};
const exportFileName = {
    github: 'github-com',
    'github-enterprise': 'github-enterprise',
    gitlab: 'gitlab',
    'bitbucket-server': 'bitbucket-server',
    'bitbucket-cloud': 'bitbucket-cloud',
};
async function generateOrgImportDataFile(source, groupId, sourceOrgId, sourceUrl, skipEmptyOrgs = false) {
    const orgData = [];
    const skippedEmptyOrgs = [];
    const topLevelEntities = await sourceGenerators[source](sourceUrl);
    await pMap(topLevelEntities, async (org) => {
        try {
            if (skipEmptyOrgs) {
                const isEmpty = await sourceNotEmpty[source](org.name, sourceUrl);
                if (isEmpty) {
                    skippedEmptyOrgs.push(org);
                    throw new Error(`Skipping empty ${exports.entityName[source]} ${org.name}`);
                }
            }
            const data = {
                name: org.name,
                groupId,
            };
            if (sourceOrgId) {
                data.sourceOrgId = sourceOrgId;
            }
            orgData.push(data);
        }
        catch (e) {
            console.warn(e.message);
        }
    }, { concurrency: 10 });
    const fileName = `group-${groupId}-${exportFileName[source]}-orgs.json`;
    await write_file_1.writeFile(fileName, {
        orgs: orgData,
    });
    return { orgs: orgData, fileName, skippedEmptyOrgs };
}
exports.generateOrgImportDataFile = generateOrgImportDataFile;
//# sourceMappingURL=generate-org-data.js.map