"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTargetsImportDataFile = void 0;
const _ = require("lodash");
const debugLib = require("debug");
const types_1 = require("../lib/types");
const write_file_1 = require("../write-file");
const lib_1 = require("../lib");
const debug = debugLib('snyk:generate-targets-data');
async function githubEnterpriseRepos(orgName, sourceUrl) {
    if (!sourceUrl) {
        console.warn('No `sourceUrl` provided for Github Enterprise source, defaulting to https://api.github.com');
    }
    const ghRepos = await lib_1.listGithubRepos(orgName, sourceUrl);
    return ghRepos;
}
const sourceGenerators = {
    [types_1.SupportedIntegrationTypesImportData.GITHUB]: lib_1.listGithubRepos,
    [types_1.SupportedIntegrationTypesImportData.GHE]: githubEnterpriseRepos,
    [types_1.SupportedIntegrationTypesImportData.GITLAB]: lib_1.listGitlabRepos,
    [types_1.SupportedIntegrationTypesImportData.AZURE_REPOS]: lib_1.listAzureRepos,
    [types_1.SupportedIntegrationTypesImportData.BITBUCKET_SERVER]: lib_1.listBitbucketServerRepos,
    [types_1.SupportedIntegrationTypesImportData.BITBUCKET_CLOUD]: lib_1.listBitbucketCloudRepos,
};
function validateRequiredOrgData(name, integrations, orgId) {
    if (!name) {
        throw new Error('Org field: `name` is required');
    }
    if (!orgId) {
        throw new Error('Org field: `orgId` is required');
    }
    if (!integrations || Object.keys(integrations).length < 1) {
        throw new Error('Org field: `integrations` is required and must have at least 1 integration');
    }
    if (_.intersection(Object.keys(integrations), Object.values(types_1.SupportedIntegrationTypesImportData)).length === 0) {
        throw new Error('At least one supported integration is expected in `integrations` field.' +
            `Supported integrations are: ${Object.values(types_1.SupportedIntegrationTypesImportData).join(',')}`);
    }
}
async function generateTargetsImportDataFile(source, orgsData, sourceUrl) {
    const targetsData = [];
    const orgsDataUnique = _.uniqBy(orgsData, 'orgId');
    for (const topLevelEntity of orgsDataUnique) {
        const { name, integrations, orgId } = topLevelEntity;
        debug(`Processing ${name}`);
        try {
            validateRequiredOrgData(name, integrations, orgId);
            const entities = await sourceGenerators[source](topLevelEntity.name, sourceUrl);
            entities.forEach((entity) => {
                targetsData.push({
                    target: entity,
                    integrationId: integrations[source],
                    orgId,
                });
            });
        }
        catch (e) {
            console.error(`Failed to generate import data for org: ${name} (${orgId}). Error: ${e.message}`);
        }
    }
    if (targetsData.length === 0) {
        throw new Error('No targets could be generated. Check the error output & try again.');
    }
    const fileName = `${source}-import-targets.json`;
    await write_file_1.writeFile(fileName, {
        targets: targetsData,
    });
    return { targets: targetsData, fileName };
}
exports.generateTargetsImportDataFile = generateTargetsImportDataFile;
//# sourceMappingURL=generate-targets-data.js.map