"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.importProjects = exports.filterOutImportedTargets = exports.shouldSkipTarget = exports.parseLogIntoTargetIds = void 0;
const debugLib = require("debug");
const path = require("path");
const snyk_request_manager_1 = require("snyk-request-manager");
const pMap = require("p-map");
const fs = require("fs");
const split = require("split");
const lib_1 = require("../lib");
const lib_2 = require("../lib");
const log_imported_batch_1 = require("../loggers/log-imported-batch");
const common_1 = require("../common");
const generate_target_id_1 = require("../generate-target-id");
const stream_data_1 = require("../stream-data");
const debug = debugLib('snyk:import-projects-script');
const debugSkipTargets = debugLib('skipping-targets');
async function parseLogIntoTargetIds(logFile) {
    return new Promise((resolve, reject) => {
        const importedTargets = [];
        fs.createReadStream(logFile)
            .pipe(split())
            .on('data', (lineObj) => {
            if (!lineObj) {
                return;
            }
            try {
                const data = JSON.parse(lineObj);
                const { orgId, integrationId, target } = data;
                importedTargets.push(generate_target_id_1.generateTargetId(orgId, integrationId, target));
            }
            catch (e) {
                console.log(e);
            }
        })
            .on('error', (err) => {
            console.error('Failed to createReadStream for file: ' + err);
            return reject(err);
        })
            .on('end', async () => {
            return resolve(importedTargets);
        });
    });
}
exports.parseLogIntoTargetIds = parseLogIntoTargetIds;
async function shouldSkipTarget(targetItem, importedTargets) {
    try {
        const { orgId, integrationId, target } = targetItem;
        const targetId = generate_target_id_1.generateTargetId(orgId, integrationId, target);
        return importedTargets.includes(targetId);
    }
    catch (e) {
        debug(`Failed to process target ${JSON.stringify(targetItem)}. ERROR: ${e}`);
        return false;
    }
}
exports.shouldSkipTarget = shouldSkipTarget;
async function filterOutImportedTargets(targets, loggingPath) {
    const filterOutImportedTargets = [];
    let importedTargets = [];
    try {
        const importedTargetsFilePath = path.resolve(process.cwd(), loggingPath, common_1.IMPORT_LOG_NAME);
        if (!fs.existsSync(importedTargetsFilePath)) {
            throw new Error(`File not found ${importedTargetsFilePath}`);
        }
        importedTargets = await parseLogIntoTargetIds(importedTargetsFilePath);
    }
    catch (e) {
        console.log(`Could not load previously imported targets file: ${common_1.IMPORT_LOG_NAME}.\nThis could be because it doesn't exist or it is malformed. Either way continuing without checking for previously imported targets.`);
        return targets;
    }
    debugSkipTargets(`Checking if any targets should be skipped`);
    await pMap(targets, async (targetItem) => {
        const shouldSkip = await shouldSkipTarget(targetItem, importedTargets);
        if (shouldSkip) {
            debugSkipTargets('Skipping target', JSON.stringify(targetItem));
        }
        else {
            filterOutImportedTargets.push(targetItem);
        }
    }, { concurrency: 150 });
    return filterOutImportedTargets;
}
exports.filterOutImportedTargets = filterOutImportedTargets;
async function importProjects(fileName, loggingPath = lib_2.getLoggingPath()) {
    var _a, _b;
    const targetsFilePath = path.resolve(process.cwd(), loggingPath, fileName);
    if (!fs.existsSync(targetsFilePath)) {
        throw new Error(`File can not be found at location ${targetsFilePath}`);
    }
    let targets = [];
    try {
        targets = (_a = (await stream_data_1.streamData(fileName, 'targets'))) !== null && _a !== void 0 ? _a : [];
    }
    catch (e) {
        throw new Error(`Failed to parse targets from ${fileName}:\n${e.message}`);
    }
    console.log(`Loaded ${targets.length} target(s) to import | ${new Date(Date.now()).toUTCString()}`);
    const concurrentTargets = lib_1.getConcurrentImportsNumber();
    const projects = [];
    console.log(`Filtering out previously imported targets, this might be slow | ${new Date(Date.now()).toUTCString()}`);
    const filteredTargets = await filterOutImportedTargets(targets, loggingPath);
    const skippedTargets = targets.length - filteredTargets.length;
    if (skippedTargets > 0) {
        console.warn(`Skipped previously imported ${skippedTargets}/${targets.length} target(s) | ${new Date(Date.now()).toUTCString()}`);
    }
    if (filteredTargets.length === 0) {
        return {
            projects: [],
            targets,
            filteredTargets,
            skippedTargets: 0,
            logFile: undefined,
        };
    }
    const requestManager = new snyk_request_manager_1.requestsManager({
        userAgentPrefix: 'snyk-api-import:import',
        period: 1000,
        maxRetryCount: 3,
    });
    let logFile;
    for (let targetIndex = 0; targetIndex < filteredTargets.length; targetIndex = targetIndex + concurrentTargets) {
        const batch = filteredTargets.slice(targetIndex, targetIndex + concurrentTargets);
        const currentTargets = skippedTargets + targetIndex + 1;
        const fullTargetsNumber = filteredTargets.length + skippedTargets;
        let currentBatchEnd = currentTargets + concurrentTargets - 1;
        if (currentBatchEnd > fullTargetsNumber) {
            currentBatchEnd = currentTargets;
        }
        const batchProgressMessages = `Importing batch ${currentTargets} - ${currentBatchEnd} out of ${fullTargetsNumber} ${skippedTargets > 0 ? `(skipped ${skippedTargets})` : ''}`;
        logFile = await log_imported_batch_1.logImportedBatch(batchProgressMessages);
        const pollingUrlsAndContext = await lib_1.importTargets(requestManager, batch, loggingPath);
        debug(`Received ${pollingUrlsAndContext.length} polling URLs`);
        const res = await lib_1.pollImportUrls(requestManager, pollingUrlsAndContext);
        debug(`Finished polling, discovered ${(_b = res.projects) === null || _b === void 0 ? void 0 : _b.length} projects`);
        projects.push(...res.projects);
    }
    return { projects, skippedTargets, filteredTargets, targets, logFile };
}
exports.importProjects = importProjects;
//# sourceMappingURL=import-projects.js.map