"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateTargets = exports.updateOrgTargets = exports.isSourceConfigured = void 0;
const pMap = require("p-map");
const debugLib = require("debug");
const path = require("path");
const fs = require("fs");
const snyk_request_manager_1 = require("snyk-request-manager");
const common_1 = require("../../common");
const lib_1 = require("../../lib");
const lib_2 = require("../../lib");
const lib_3 = require("../../lib");
const feature_flags_1 = require("../../lib/api/feature-flags");
const types_1 = require("../../lib/types");
const log_updated_project_1 = require("../../loggers/log-updated-project");
const sync_projects_per_target_1 = require("./sync-projects-per-target");
const log_failed_sync_1 = require("../../loggers/log-failed-sync");
const log_failed_to_update_projects_1 = require("../../loggers/log-failed-to-update-projects");
const debug = debugLib('snyk:sync-org-projects');
function isSourceConfigured(origin) {
    const getDefaultBranchGenerators = {
        [types_1.SupportedIntegrationTypesUpdateProject.GITHUB]: lib_2.isGithubConfigured,
        [types_1.SupportedIntegrationTypesUpdateProject.GHE]: lib_2.isGithubConfigured,
    };
    return getDefaultBranchGenerators[origin];
}
exports.isSourceConfigured = isSourceConfigured;
async function updateOrgTargets(publicOrgId, sources, sourceUrl, config = {
    dryRun: false,
    entitlements: ['openSource'],
    exclusionGlobs: [],
}) {
    const res = {
        failedTargets: 0,
        processedTargets: 0,
        meta: {
            projects: {
                updated: [],
                failed: [],
            },
        },
    };
    // ensure source is enabled for sync
    const allowedSources = sources.filter((source) => Object.values(types_1.SupportedIntegrationTypesUpdateProject).includes(source));
    if (!allowedSources.length) {
        throw new Error(`Nothing to sync, stopping. Sync command currently only supports the following sources: ${Object.values(types_1.SupportedIntegrationTypesUpdateProject).join(',')}`);
    }
    const requestManager = new snyk_request_manager_1.requestsManager({
        userAgentPrefix: 'snyk-api-import:sync',
        period: 1000,
        maxRetryCount: 3,
    });
    let hasCustomBranchFlag = true;
    try {
        hasCustomBranchFlag = await feature_flags_1.getFeatureFlag(requestManager, 'customBranch', publicOrgId);
    }
    catch (e) {
        throw new Error(`Org ${publicOrgId} was not found or you may not have the correct permissions to access the org`);
    }
    // TODO: move this into sync project per target and skip only whats needed
    if (hasCustomBranchFlag) {
        throw new Error(`Detected custom branches feature. Skipping syncing organization ${publicOrgId} because it is not possible to determine which should be the default branch.`);
    }
    await pMap(allowedSources, async (source) => {
        isSourceConfigured(source)();
        const filters = {
            limit: 100,
            origin: source,
            excludeEmpty: true,
        };
        console.log(`Listing all targets for source ${source}`);
        const { targets } = await lib_3.listTargets(requestManager, publicOrgId, filters);
        console.log(`Resolving integration ID for source ${source}`);
        const integrationsData = await lib_1.listIntegrations(requestManager, publicOrgId);
        const integrationId = integrationsData[source];
        console.log(`Syncing targets for source ${source}`);
        const response = await updateTargets(requestManager, publicOrgId, targets, integrationId, sourceUrl, config);
        console.log(`Done syncing targets for source ${source}`);
        res.processedTargets += response.processedTargets;
        res.failedTargets += response.failedTargets;
        res.meta.projects.updated.push(...response.meta.projects.updated);
        res.meta.projects.failed.push(...response.meta.projects.failed);
    }, { concurrency: 30 });
    const failedLogExists = fs.existsSync(`${lib_3.getLoggingPath()}/${publicOrgId}.${common_1.FAILED_UPDATE_PROJECTS_LOG_NAME}`);
    return {
        ...res,
        fileName: path.resolve(`${lib_3.getLoggingPath()}/${publicOrgId}.${common_1.UPDATED_PROJECTS_LOG_NAME}`),
        failedFileName: failedLogExists
            ? path.resolve(`${lib_3.getLoggingPath()}/${publicOrgId}.${common_1.FAILED_UPDATE_PROJECTS_LOG_NAME}`)
            : undefined,
    };
}
exports.updateOrgTargets = updateOrgTargets;
async function updateTargets(requestManager, orgId, targets, integrationId, sourceUrl, config = {
    dryRun: false,
    entitlements: ['openSource'],
}) {
    let processedTargets = 0;
    let failedTargets = 0;
    const updatedProjects = [];
    const failedProjects = [];
    const loggingPath = lib_3.getLoggingPath();
    const concurrentTargets = 30;
    await pMap(targets, async (target) => {
        try {
            console.log(`Processing target ${target.attributes.displayName}`);
            // TODO: is target reachable via SCM token? If not skip listing projects
            const { updated, failed } = await sync_projects_per_target_1.syncProjectsForTarget(requestManager, orgId, target, integrationId, sourceUrl, config);
            updatedProjects.push(...updated);
            failedProjects.push(...failed);
            processedTargets += 1;
            if (updated.length) {
                await log_updated_project_1.logUpdatedProjects(orgId, updated);
            }
            if (failed.length) {
                await log_failed_to_update_projects_1.logFailedToUpdateProjects(orgId, failed);
            }
        }
        catch (e) {
            failedTargets += 1;
            debug(e);
            const errorMessage = e.message;
            console.warn(`Failed to sync target ${target.attributes.displayName}. ERROR: ${errorMessage}`);
            await log_failed_sync_1.logFailedSync(orgId, target, errorMessage, loggingPath);
        }
        finally {
            console.log(`Finished processing target ${target.attributes.displayName}`);
        }
        // TODO: exit early if 100% of batch targets failed.
    }, { concurrency: concurrentTargets });
    return {
        processedTargets,
        // TODO: collect failed targets & log them with reason?
        failedTargets,
        meta: {
            projects: {
                updated: updatedProjects,
                failed: failedProjects,
            },
        },
    };
}
exports.updateTargets = updateTargets;
//# sourceMappingURL=sync-org-projects.js.map