"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.snykTargetConverter = exports.bulkImportTargetFiles = exports.bulkDeactivateProjects = exports.bulkUpdateProjectsBranch = exports.syncProjectsForTarget = exports.getTargetConverter = exports.getMetaDataGenerator = void 0;
const debugLib = require("debug");
const github_1 = require("../../lib/source-handlers/github");
const update_branch_1 = require("../../lib/project/update-branch");
const types_1 = require("../../lib/types");
const types_2 = require("../../lib/types");
const lib_1 = require("../../lib");
const pMap = require("p-map");
const clone_and_analyze_1 = require("./clone-and-analyze");
const import_target_1 = require("./import-target");
const debug = debugLib('snyk:sync-projects-per-target');
function getMetaDataGenerator(origin) {
    const getDefaultBranchGenerators = {
        [types_2.SupportedIntegrationTypesUpdateProject.GITHUB]: github_1.getGithubRepoMetaData,
        [types_2.SupportedIntegrationTypesUpdateProject.GHE]: github_1.getGithubRepoMetaData,
    };
    return getDefaultBranchGenerators[origin];
}
exports.getMetaDataGenerator = getMetaDataGenerator;
function getTargetConverter(origin) {
    const getTargetConverter = {
        [types_2.SupportedIntegrationTypesUpdateProject.GITHUB]: snykTargetConverter,
        [types_2.SupportedIntegrationTypesUpdateProject.GHE]: snykTargetConverter,
    };
    return getTargetConverter[origin];
}
exports.getTargetConverter = getTargetConverter;
async function syncProjectsForTarget(requestManager, orgId, target, integrationId, host, config = {
    dryRun: false,
    entitlements: ['openSource'],
}) {
    const updated = new Set();
    const failed = new Set();
    debug(`Listing projects for target ${target.attributes.displayName}`);
    const { projects } = await lib_1.listProjects(requestManager, orgId, {
        targetId: target.id,
        limit: 100,
    });
    debug(`Syncing projects for target ${target.attributes.displayName}`);
    let targetMeta;
    const origin = target.attributes
        .origin;
    const targetData = getTargetConverter(origin)(target);
    try {
        targetMeta = await getMetaDataGenerator(origin)(targetData, host);
    }
    catch (e) {
        //TODO: if repo is deleted, deactivate all projects
        debug(e);
        const error = `Getting default branch via ${origin} API failed with error: ${e.message}`;
        projects.map((project) => {
            failed.add({
                errorMessage: error,
                projectPublicId: project.id,
                type: types_1.ProjectUpdateType.BRANCH,
                from: project.branch,
                to: targetMeta.branch,
                dryRun: config.dryRun,
                target,
            });
        });
    }
    const deactivate = [];
    const createProjects = [];
    if (targetMeta.archived) {
        deactivate.push(...projects);
    }
    else {
        try {
            const res = await clone_and_analyze_1.cloneAndAnalyze(origin, targetMeta, projects, {
                exclusionGlobs: config.exclusionGlobs,
                entitlements: config.entitlements,
                manifestTypes: config.manifestTypes,
            });
            debug('Analysis finished', JSON.stringify({
                remove: res.remove.length,
                import: res.import.length,
            }));
            deactivate.push(...res.remove);
            createProjects.push(...res.import);
        }
        catch (e) {
            debug(e);
            const error = `Cloning and analysing the repo to deactivate projects failed with error: ${e.message}`;
            projects.map((project) => {
                failed.add({
                    errorMessage: error,
                    projectPublicId: project.id,
                    from: 'active',
                    to: 'deactivated',
                    type: types_1.ProjectUpdateType.DEACTIVATE,
                    dryRun: config.dryRun,
                    target,
                });
            });
        }
    }
    // remove any projects that are to be deactivated from other actions
    const branchUpdateProjects = [];
    const deactivateIds = deactivate.map((p) => p.id);
    for (const p of projects.filter((p) => p.status !== 'inactive')) {
        if (!deactivateIds.includes(p.id)) {
            branchUpdateProjects.push(p);
        }
    }
    const actions = [
        bulkUpdateProjectsBranch(requestManager, orgId, branchUpdateProjects, targetMeta.branch, config.dryRun),
        bulkDeactivateProjects(requestManager, orgId, deactivate, config.dryRun),
        bulkImportTargetFiles(requestManager, orgId, createProjects, integrationId, { ...targetData, branch: targetMeta.branch }, config.dryRun),
    ];
    const [branchUpdate, deactivatedProjects, newProjects] = await Promise.all(actions);
    [
        ...branchUpdate.updated,
        ...deactivatedProjects.updated,
        ...newProjects.updated,
    ]
        .map((t) => ({ ...t, target }))
        .forEach((i) => updated.add(i));
    [...branchUpdate.failed, ...deactivatedProjects.failed, ...newProjects.failed]
        .map((t) => ({ ...t, target }))
        .forEach((i) => failed.add(i));
    // TODO: add failed target info for logs
    return {
        updated: Array.from(updated),
        failed: Array.from(failed),
    };
}
exports.syncProjectsForTarget = syncProjectsForTarget;
async function bulkUpdateProjectsBranch(requestManager, orgId, projects, branch, dryRun = false) {
    const updatedProjects = [];
    const failedProjects = [];
    await pMap(projects, async (project) => {
        try {
            const { updated } = await update_branch_1.updateBranch(requestManager, {
                branch: project.branch,
                projectPublicId: project.id,
            }, branch, orgId, dryRun);
            if (updated) {
                updatedProjects.push({
                    projectPublicId: project.id,
                    type: types_1.ProjectUpdateType.BRANCH,
                    from: project.branch,
                    to: branch,
                    dryRun,
                });
                debug(`Default branch updated from ${project.branch} to ${branch} for ${project.id}`);
            }
        }
        catch (e) {
            failedProjects.push({
                errorMessage: e.message,
                projectPublicId: project.id,
                type: types_1.ProjectUpdateType.BRANCH,
                from: project.branch,
                to: branch,
                dryRun,
            });
        }
    }, { concurrency: 50 });
    return { updated: updatedProjects, failed: failedProjects };
}
exports.bulkUpdateProjectsBranch = bulkUpdateProjectsBranch;
async function bulkDeactivateProjects(requestManager, orgId, projects = [], dryRun = false) {
    const updated = [];
    const failed = [];
    if (!projects.length) {
        return { updated, failed };
    }
    debug(`De-activating projects: ${projects.map((p) => p.id).join(',')}`);
    await pMap(projects, async (project) => {
        if (!dryRun) {
            try {
                await lib_1.deactivateProject(requestManager, orgId, project.id);
                updated.push({
                    projectPublicId: project.id,
                    type: types_1.ProjectUpdateType.DEACTIVATE,
                    from: 'active',
                    to: 'deactivated',
                    dryRun,
                });
            }
            catch (e) {
                failed.push({
                    projectPublicId: project.id,
                    type: types_1.ProjectUpdateType.DEACTIVATE,
                    from: 'active',
                    to: 'deactivated',
                    dryRun,
                    errorMessage: `Could not deactivate project: ${e.message}`,
                });
            }
        }
        else {
            debug('Skipping de-activating project in dryRun mode');
            updated.push(...projects.map((p) => ({
                projectPublicId: p.id,
                type: types_1.ProjectUpdateType.DEACTIVATE,
                from: 'active',
                to: 'deactivated',
                dryRun,
            })));
        }
    }, { concurrency: 50 });
    return { updated, failed };
}
exports.bulkDeactivateProjects = bulkDeactivateProjects;
async function bulkImportTargetFiles(requestManager, orgId, files = [], integrationId, target, dryRun = false, concurrentFilesImport = 30) {
    const updated = [];
    const failed = [];
    if (!files.length) {
        return { updated, failed };
    }
    debug(`Importing ${files.length} files`);
    if (!dryRun) {
        for (let index = 0; index < files.length; index = index + concurrentFilesImport) {
            const batch = files.slice(index, index + concurrentFilesImport);
            const { projects, failed: failedProjects } = await import_target_1.importSingleTarget(requestManager, orgId, integrationId, target, batch);
            projects.map((p) => {
                var _a, _b;
                const projectId = (_a = p.projectUrl) === null || _a === void 0 ? void 0 : _a.split('/').slice(-1)[0];
                updated.push({
                    projectPublicId: projectId,
                    type: types_1.ProjectUpdateType.IMPORT,
                    from: (_b = p.targetFile) !== null && _b !== void 0 ? _b : '',
                    to: p.projectUrl,
                    dryRun,
                });
            });
            failedProjects.map((f) => {
                var _a, _b;
                failed.push({
                    projectPublicId: '',
                    type: types_1.ProjectUpdateType.IMPORT,
                    from: (_a = f.targetFile) !== null && _a !== void 0 ? _a : '',
                    to: f.projectUrl,
                    dryRun,
                    errorMessage: (_b = f.userMessage) !== null && _b !== void 0 ? _b : 'Failed to import project via Snyk Import API',
                });
            });
        }
        return { updated, failed };
    }
    else {
        files.map((f) => updated.push({
            projectPublicId: '',
            type: types_1.ProjectUpdateType.IMPORT,
            from: f,
            to: `https://app.snyk.io/org/example-org-name/project/example-project-id-uuid`,
            dryRun,
        }));
        return { updated, failed: [] };
    }
}
exports.bulkImportTargetFiles = bulkImportTargetFiles;
function snykTargetConverter(target) {
    const [owner, name] = target.attributes.displayName.split('/');
    return {
        owner,
        name: name,
    };
}
exports.snykTargetConverter = snykTargetConverter;
//# sourceMappingURL=sync-projects-per-target.js.map